cdef class Frame:

    def __init__(self, name: str):
        """
        Constructor of Frame class which takes inputStream as input and reads the frame
        :param name: Name of the frame
        """
        self.__lexical_units = []
        self.__frame_elements = []
        self.__name = name

    cpdef addLexicalUnit(self, str lexicalUnit):
        """
        Adds a new lexical unit to the current frame
        :param lexicalUnit: lexicalUnit Lexical unit to be added
        """
        self.__lexical_units.append(lexicalUnit)

    cpdef addFrameElement(self, str frameElement):
        """
        Adds a new frame element to the current frame
        :param frameElement: Frame element to be added
        """
        self.__frame_elements.append(frameElement)

    cpdef bint lexicalUnitExists(self, str synSetId):
        """
        Checks if the given lexical unit exists in the current frame
        :param synSetId: Lexical unit to be searched.
        :return: True if the lexical unit exists, false otherwise.
        """
        return synSetId in self.__lexical_units

    cpdef str getLexicalUnit(self, int index):
        """
        Accessor for a given index in the lexicalUnit array.
        :param index: Index of the lexical unit
        :return: The lexical unit at position index in the lexicalUnit array
        """
        return self.__lexical_units[index]

    cpdef str getFrameElement(self, int index):
        """
        Accessor for a given index in the frameElements array.
        :param index: Index of the frame element
        :return: The frame element at position index in the frameElements array
        """
        return self.__frame_elements[index]

    cpdef int lexicalUnitSize(self):
        """
        Returns number of lexical units in the current frame
        :return: Number of lexical units in the current frame
        """
        return len(self.__lexical_units)

    cpdef int frameElementSize(self):
        """
        Returns number of frame elements in the current frame
        :return: Number of frame elements in the current frame
        """
        return len(self.__frame_elements)

    cpdef str getName(self):
        """
        Accessor for the name of the frame
        :return: Name of the frame
        """
        return self.__name

    def __repr__(self):
        return f"{self.__name} {self.__lexical_units} {self.__frame_elements}"



cdef class FrameElementList(object):

    cdef list __frame_elements

    cpdef updateConnectedId(self, str previousId, str currentId)
    cpdef addPredicate(self, str predicateId)
    cpdef removePredicate(self)
    cpdef bint containsPredicate(self)
    cpdef bint containsPredicateWithId(self, str predicateId)
    cpdef list getFrameElements(self)
    cpdef bint containsFrameElement(self, str argumentType, str frame, str _id)



from FrameNet.FrameElement cimport FrameElement


cdef class FrameElementList(object):

    def __init__(self, frameElementList: str):
        """
        Constructor of frame element list from a string. The frame elements for a word is a concatenated list of
        frame element separated via '#' character.
        :param frameElementList: String consisting of frame elements separated with '#' character.
        """
        cdef str item
        self.__frame_elements = []
        items = frameElementList.split('#')
        for item in items:
            self.__frame_elements.append(FrameElement(item))

    def __str__(self) -> str:
        """
        Overloaded toString method to convert a frame element list to a string. Concatenates the string forms of all
        frame element with '#' character.
        :return: String form of the frame element list.
        """
        cdef int i
        cdef str result
        if len(self.__frame_elements) == 0:
            return "NONE"
        else:
            result = self.__frame_elements[0].__str__()
            for i in range(1, len(self.__frame_elements)):
                result += "#" + self.__frame_elements[i].__str__()
            return result

    cpdef updateConnectedId(self, str previousId, str currentId):
        """
        Replaces id's of predicates, which have previousId as synset id, with currentId.
        :param previousId: Previous id of the synset.
        :param currentId: Replacement id.
        """
        cdef FrameElement frame_element
        for frame_element in self.__frame_elements:
            if frame_element.getId() == previousId:
                frame_element.setId(currentId)

    cpdef addPredicate(self, str predicateId):
        """
        Adds a predicate argument to the frame element list of this word.
        :param predicateId: Synset id of this predicate.
        """
        if len(self.__frame_elements) != 0 and self.__frame_elements[0].getFrameElementType() == "NONE":
            self.__frame_elements.pop(0)
        self.__frame_elements.append(FrameElement("PREDICATE", "NONE", predicateId))

    cpdef removePredicate(self):
        """
        Removes the predicate with the given predicate id.
        """
        cdef FrameElement frame_element
        for frame_element in self.__frame_elements:
            if frame_element.getFrameElementType() == "PREDICATE":
                self.__frame_elements.remove(frame_element)
                break

    cpdef bint containsPredicate(self):
        """
        Checks if one of the frame elements is a predicate.
        :return: True, if one of the frame elements is predicate; false otherwise.
        """
        cdef FrameElement frame_element
        for frame_element in self.__frame_elements:
            if frame_element.getFrameElementType() == "PREDICATE":
                return True
        return False

    cpdef bint containsPredicateWithId(self, str predicateId):
        """
        Checks if one of the frame element is a predicate with the given id.
        :param predicateId: Synset id to check.
        :return: True, if one of the frame element is predicate; false otherwise.
        """
        cdef FrameElement frame_element
        for frame_element in self.__frame_elements:
            if frame_element.getFrameElementType() == "PREDICATE" and frame_element.getId() == predicateId:
                return True
        return False

    cpdef list getFrameElements(self):
        """
        Returns the frame elements as an array list of strings.
        :return: Frame elements as an array list of strings.
        """
        cdef FrameElement frame_element
        cdef list result
        result = []
        for frame_element in self.__frame_elements:
            result.append(frame_element.__str__())
        return result

    cpdef bint containsFrameElement(self,
                             str frameElementType,
                             str frame,
                             str _id):
        """
        Checks if the given argument with the given type and id exists or not.
        :param frameElementType: Type of the frame element to search for.
        :param frame: Name of the frame to search for
        :param _id: Id of the frame element to search for.
        :return: True if the frame element exists, false otherwise.
        """
        cdef FrameElement frame_element
        for frame_element in self.__frame_elements:
            if frame_element.getFrameElementType() == frameElementType and frame_element.getFrame() == frame and frame_element.getId() == _id:
                return True
        return False



cdef class FrameElement(object):

    cdef str __frame_element_type
    cdef str __frame
    cdef str __id

    cpdef constructor1(self, str frameElement)
    cpdef constructor2(self, str frameElementType, str frame, str _id)
    cpdef str getFrameElementType(self)
    cpdef str getFrame(self)
    cpdef str getId(self)
    cpdef setId(self, str _id)



cdef class Frame:

    cdef str __name
    cdef list __lexical_units
    cdef list __frame_elements

    cpdef addLexicalUnit(self, str lexicalUnit)
    cpdef addFrameElement(self, str frameElement)
    cpdef bint lexicalUnitExists(self, str synSetId)
    cpdef str getLexicalUnit(self, int index)
    cpdef str getFrameElement(self, int index)
    cpdef int lexicalUnitSize(self)
    cpdef int frameElementSize(self)
    cpdef str getName(self)



import xml.etree.ElementTree

import pkg_resources

cdef class FrameNet:

    def __init__(self):
        """
        A constructor of FrameNet class which reads all frame files inside the files2.txt file. For each filename inside
        that file, the constructor creates a FrameNet.Frame and puts in inside the frames array.
        """
        self.__frames = []
        root = xml.etree.ElementTree.parse(pkg_resources.resource_filename(__name__, 'data/framenet.xml')).getroot()
        for frame_node in root:
            frame = Frame(frame_node.attrib["NAME"])
            for child_node in frame_node:
                if child_node.tag == "LEXICAL_UNITS":
                    for lexical_unit in child_node:
                        frame.addLexicalUnit(lexical_unit.text)
                elif child_node.tag == "FRAME_ELEMENTS":
                    for frame_element in child_node:
                        frame.addFrameElement(frame_element.text)
            self.__frames.append(frame)

    cpdef bint lexicalUnitExists(self, str synSetId):
        """
        Checks if the given lexical unit exists in any frame in the frame set.
        :param synSetId: Id of the lexical unit
        :return: True if any frame contains the given lexical unit, false otherwise.
        """
        cdef Frame frame
        for frame in self.__frames:
            if frame.lexicalUnitExists(synSetId):
                return True
        return False

    cpdef list getFrames(self, str synSetId):
        """
        Returns an array of frames that contain the given lexical unit in their lexical units
        :param synSetId: Id of the lexical unit.
        :return: An array of frames that contains the given lexical unit.
        """
        cdef list result
        cdef Frame frame
        result = []
        for frame in self.__frames:
            if frame.lexicalUnitExists(synSetId):
                result.append(frame)
        return result

    cpdef int size(self):
        """
        Returns number of frames in the frame set.
        :return: Number of frames in the frame set.
        """
        return len(self.__frames)

    cpdef Frame getFrame(self, int index):
        """
        Returns the element at the specified position in the frame list.
        :param index: index of the element to return
        :return: The element at the specified position in the frame list.
        """
        return self.__frames[index]
